#!/usr/bin/env python
# To use:
#       python setup.py install
#
import re
import build_config
import os, sys, string, re
from glob import glob

import numpy    
import distutils
from distutils.command.install import install
from distutils.core import setup, Extension


def uniq(input):
  output = []
  for x in input:
    if x not in output:
      output.append(x)
  return output

headers = glob (os.path.join ("*.h") ) + [ "hst_cpu/hst_cpu.h", "hst_cuda/hst_cuda.h", "hst_opencl/hst_opencl.h" ];
header = headers + glob (os.path.join ("Include/Numeric","*.h") ) 
objects = ['PyHST_c.c',"Vhst_calculate_limits.c", "hst.c", "hst_setup.c", "hst_reconstructor.c", "debug.c", "hw_sched.c", "hw_thread.c"]

if hasattr(build_config, 'CFLAGS'):
    os.environ["CFLAGS"]=build_config.CFLAGS

if hasattr(build_config, 'INCLUDES'):
    incdirs = build_config.INCLUDES
else:
    incdirs = []

if hasattr(build_config, 'DEFINES'):
    defines = build_config.DEFINES
else:
    defines = []
    
if hasattr(build_config, 'LIBDIRS'):
    libdirs = build_config.LIBDIRS
else:
    libdirs = []

if hasattr(build_config, 'LIBS'):
    libs = build_config.LIBS
else:
    libs = []

regexp = re.compile("^-D")
for i, define in enumerate(defines):
    defines[i] = (regexp.sub("", define), 1);
    
for i, lib in enumerate(libs):
    m = re.search("^(.*)/([^/]+)$", lib)
    if m:
	path = m.group(1)
	lib = m.group(2)

	m = re.search("^(.*)\\.[^.]+$", lib)
	if m:
	    lib = m.group(1)

	m = re.search("^lib(.*)$", lib)
	if m:
	    lib = m.group(1)
	
	libs[i] = lib
	libdirs.append(path)


libs += ['gcc']
libdirs += ['./']

# The version is set in Lib/numeric_version.py

gpu_build, opencl_build = False, False
for arg in sys.argv:
    if re.search("USEGPU", arg):
        gpu_build = True
        break
    elif re.search("USEOPENCL", arg):
        opencl_build = True

if gpu_build:
    libdirs += ["hst_cuda"]
    libs += ["hst_cuda"]

if opencl_build:
    libdirs += ["hst_opencl"]
    libs += ["hst_opencl"]

#if("PROFILE" in sys.argv):
#    libs.extend(['c_p', 'm_p'])
#else:
libs.append('m')
    
	
setup (name = "extension",
       description = " Extension to Python",
       include_dirs = numpy.get_include(),
       ext_modules = [Extension('PyHST_c',
                                objects,
				depends = header + ["build_config.py"],
				define_macros = uniq(defines),
                                libraries = uniq(libs),
                                library_dirs  = uniq(libdirs),
				include_dirs = uniq(incdirs),
                                ),
                     ]

       )
